@@ -1,12 +1,10 @@
 # AltaVista.pm
 # by John Heidemann
 # Copyright (C) 1996-1998 by USC/ISI
-# $Id: AltaVista.pm,v 1.9 2002/08/21 13:21:55 mthurn Exp $
+# $Id: AltaVista.pm,v 2.28 2003-03-30 17:35:56-05 kingpin Exp kingpin $
 #
 # Complete copyright notice follows below.
 
-package WWW::Search::AltaVista;
-
 =head1 NAME
 
 WWW::Search::AltaVista - class for searching www.altavista.com
@@ -30,41 +28,74 @@ be done through WWW::Search objects.
 
 =head1 OPTIONS
 
-The default is for simple web queries.
-Specialized back-ends for simple and advanced web and news searches
-are available (see
-L<WWW::Search::AltaVista::Web>,
-L<WWW::Search::AltaVista::AdvancedWeb>,
-L<WWW::Search::AltaVista::News>,
-L<WWW::Search::AltaVista::AdvancedNews>).
-These back-ends set different combinations following options.
+The default is "any of these words" (OR of query terms).
 
 =over 8
 
-=item search_url=URL
+=item aqa=all+of+these+words
 
-Specifies whom to query with the AltaVista protocol.
-The default is at
-C<http://www.altavista.com/cgi-bin/query>;
-you may wish to retarget it to
-C<http://www.altavista.telia.com/cgi-bin/query>
-or other hosts if you think that they're ``closer''.
+Add the AND of these words to the query.
 
-=item search_debug, search_parse_debug, search_ref
-Specified at L<WWW::Search>.
+=item aqp=this+exact+phrase
 
-=item pg=aq
+Add "this exact phrase" to the query.
 
-Do advanced queries.
-(It defaults to simple queries.)
+=item aqo=any+of+these+words
 
-=item what=news
+Add the OR of these words to the query.
+This is where the query is placed by default.
 
-Search Usenet instead of the web.
-(It defaults to search the web.)
+=item aqn=none+of+these+words
+
+Add NOT these words to the query.
+
+=item aqb=(boolean+AND+expression)+NEAR+entry
+
+Add a boolean expression to the query.
+Operators are AND, OR, AND NOT, and NEAR.
+In the browser interface, the boolean expression can not be combined with any other query types listed above.
+You should probably build the boolean expression with parentheses and spaces and urlescape it.
+
+=item aqs=these+words
+
+Pages containing "these words" will be ranked highest.
+
+=item kgs=[0,1]
+
+To restrict the search to U.S. websites, set kgs=1.
+The default is world-wide, kgs=0.
+
+=item kls=[0,1]
+
+To restrict the search to pages in English and Spanish, set kls=1.
+The default is no language restrictions, kls=0.
+
+=item filetype=[html,pdf]
+
+To restrict the search to HTML pages only, set filetype=html.
+To restrict the search to PDF pages only, set filetype=pdf.
+The default is no restriction on page type, filetype=.
+
+=item rc=dmn&swd=net+org+or.jp
+
+To restrict the search to pages from certain domains,
+set rc=dmn and set swd to a list of desired toplevel domains.
+
+=item rc=url&lh=www.sandcrawler.com/SWB
+
+To restrict the search to pages from a particular site,
+set rc=url and set lh to the site name and path.
+Leave off the http:// from the site.
 
 =back
 
+=head1 BUGS
+
+=over
+
+=item Not all of the above options have been tested.
+
+=back
 
 =head1 SEE ALSO
 
@@ -101,15 +132,22 @@ MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
 #####################################################################
 
-require Exporter;
+package WWW::Search::AltaVista;
+
+use Carp ();
+use Exporter;
+use WWW::Search qw( generic_option strip_tags unescape_query );
+use WWW::Search::Result;
+
+use strict;
+
+use vars qw( @ISA @EXPORT @EXPORT_OK $VERSION $MAINTAINER );
+
 @EXPORT = qw();
 @EXPORT_OK = qw();
 @ISA = qw(WWW::Search Exporter);
-$VERSION = '2.26';
-
-use Carp ();
-use WWW::Search qw( generic_option unescape_query );
-require WWW::SearchResult;
+$MAINTAINER = 'Martin Thurn <mthurn@cpan.org>';
+$VERSION = sprintf("%d.%02d", q$Revision: 2.28 $ =~ /(\d+)\.(\d+)/o);
 
 
 sub undef_to_emptystring
@@ -118,24 +156,53 @@ sub undef_to_emptystring
   }
 
 
+sub gui_query
+  {
+  my ($self, $sQuery, $rh) = @_;
+  $self->{'_options'} = {
+                         'search_host' => 'http://www.altavista.com',
+                         'search_path' => '/web/results',
+                         'q' => $sQuery,
+                         'kls' => 0,
+                         avkw => 'qtrp',
+                        };
+  return $self->native_query($sQuery, $rh);
+  } # gui_query
+
+
 # private
 sub native_setup_search
   {
-  my($self, $native_query, $native_options_ref) = @_;
+  my ($self, $native_query, $native_options_ref) = @_;
   $self->user_agent('user');
   $self->{_next_to_retrieve} = 0;
-  # set the text=yes option to provide next links with <a href>
-  # (suggested by Guy Decoux <decoux@moulon.inra.fr>).
   if (!defined($self->{_options}))
     {
     $self->{_options} = {
-                         'pg' => 'q',
-                         'text' => 'yes',
-                         'what' => 'web',
-                         'fmt' => 'd',
+                         'pg' => 'aq',
+                         'avkw' => 'qtrp',
+                         'aqmode' => 's',
+                         'aqo' => $native_query,
+                         'kgs' => 0,
+                         'kls' => 0,
+                         # 'dt' => 'dtrange',
+                         'rc' => 'dmn',
                          'nbq' => '50',
-                         'search_url' => 'http://www.altavista.com/cgi-bin/query',
+                         'search_host' => 'http://www.altavista.com',
+                         'search_path' => '/web/results',
                         };
+    if ((my $s = $self->date_from) ne '')
+      {
+      $s = &UnixDate($s, '%m/%d/%y');
+      $self->{_options}->{d0} = $s;
+      $self->{_options}->{dt} = 'dtrange';
+      } # if
+    if ((my $s = $self->date_to) ne '')
+      {
+      $s = &UnixDate($s, '%m/%d/%y');
+      $self->{_options}->{d1} = $s;
+      $self->{_options}->{dt} = 'dtrange';
+      } # if
     } # if
   my($options_ref) = $self->{_options};
   if (defined($native_options_ref))
@@ -160,6 +227,7 @@ sub native_setup_search
     next if $key eq 'mss';
     $options .= $key . '=' . $options_ref->{$key} . '&';
     } # foreach
+  chop $options;
   $self->{_debug} = $options_ref->{'search_debug'};
   $self->{_debug} = 2 if ($options_ref->{'search_parse_debug'});
   $self->{_debug} = 0 if (!defined($self->{_debug}));
@@ -167,8 +235,8 @@ sub native_setup_search
   # Finally figure out the url.
   $self->{_base_url} =
   $self->{_next_url} =
-  $self->{_options}{'search_url'} .'?'. $options .'q='. $native_query;
-  print STDERR $self->{_base_url} . "\n" if ($self->{_debug});
+  $self->{_options}{'search_host'} . $self->{_options}{'search_path'} .'?'. $options;
+  # print STDERR $self->{_base_url} . "\n" if ($self->{_debug});
   } # native_setup_search
 
 
@@ -212,35 +280,34 @@ sub native_retrieve_some
     print STDERR "WWW::Search::AltaVista::native_retrieve_some: fetching " . $self->{_next_url} . "\n" if ($self->{_debug});
     my($response) = $self->http_request('GET', $self->{_next_url});
     $self->{response} = $response;
-    if (!$response->is_success)
+    if (! $response->is_success)
       {
-      print STDERR " +   failed: $response\n" if ($self->{_debug});
+      print STDERR " +   failed: ", $response->as_string if ($self->{_debug});
       return undef;
       } # if
 
     # parse the output
-    my($HEADER, $HITS, $INHIT, $TRAILER, $POST_NEXT) = (1..10);  # order matters
-    my($hits_found) = 0;
-    my($state) = ($HEADER);
-    my($hit) = undef;
-    my($raw) = '';
+    my ($HEADER, $HITS, $INHIT, $TRAILER, $POST_NEXT) = (1..10);  # order matters
+    my $hits_found = 0;
+    my $state = $HEADER;
+    my $hit = undef;
+    my $raw = '';
     foreach ($self->split_lines($response->content()))
       {
-      next if m@^$@; # short circuit for blank lines
+      next if m/^$/; # short circuit for blank lines
       print STDERR "PARSE(0:RAW): $_\n" if ($self->{_debug} >= 3);
-      if (0) { }
 
       ######
       # HEADER PARSING: find the number of hits
       #
-      elsif ($state == $HEADER && /(?:AltaVista|We)\s+found.*?([\d,]+)\s+(results?|headlines?)/i)
+      if ($state == $HEADER && /(?:AltaVista|We)\s+found.*?([\d,]+)\s+(results?|headlines?)/i)
         {
         # Modified by Jim
-        my($n) = $1;
+        my $n = $1;
         $n =~ s/,//g;
         $self->approximate_result_count($n);
         print STDERR "PARSE(10:HEADER->HITS): $n documents found.\n" if ($self->{_debug} >= 2);
-        return 0 unless 0 < $n;
+        return 0 unless (0 < $n);
         $state = $HITS;
         }
 
@@ -251,50 +318,66 @@ sub native_retrieve_some
         {
         $raw .= $_;
         my ($url, $title) = (unescape_query($1), $2);
-        ($hit, $raw) = $self->begin_new_hit($hit, $raw);
-        $hits_found++;
-        $hit->add_url($url);
-        $hit->title($title);
         print STDERR "PARSE(13:INHIT): url+title: $title.\n" if ($self->{_debug} >= 2);
-    } elsif ($state == $HITS && /^<br>/i) {
-    } elsif ($state == $HITS && /^([\d|\w|<b>|\.].+)<br>/i)
-      {
-      # We are looking at a description...
-      if (ref $hit)
+        if ($title !~ m!\A<img!i)
+          {
+          ($hit, $raw) = $self->begin_new_hit($hit, $raw);
+          $hits_found++;
+          $hit->add_url($url);
+          $hit->title($title);
+          } # if
+        }
+      elsif ($state == $HITS && /^URL:\s(.*)$/i)
         {
-        # AND we have already seen a URL.
-        $raw .= $_;
-        $hit->description($1);
-        } # if
-      print STDERR "PARSE(13:INHIT): description.\n" if ($self->{_debug} >= 2);
-      }
-    elsif ($state == $HITS && /^URL:\s(.*)$/i) { #"
         $raw .= $_;
         print STDERR "PARSE(13:INHIT): url: $1.\n" if ($self->{_debug} >= 2);
+        }
 
+      if (0 && $state == $HITS && /^<br>/i)
+        {
+        }
+
+      if (($state == $HITS) &&
+          (m/^([\d|\w|<b>|\.].+?)<br>/i ||
+           m!<span\s+class=s>(.+?)<br>!i))
+        {
+        # We are looking at a description...
+        if (ref $hit)
+          {
+          # AND we have already seen a URL.
+          $raw .= $_;
+          $hit->description(&strip_tags($1));
+          } # if
+        print STDERR "PARSE(13:INHIT): description.\n" if ($self->{_debug} >= 2);
         }
       # Look for end of hits list:
-      elsif (($state == $HITS)
-             &&
-             (
-              /^<!-- res_extend.wm -->/i
-              ||
-              m!<a\s+href="/r\?ext24"!i
-              ||
-              m!<a\s+href="http://jump.altavista.com/rlweb_ebay.go!i
-             )
-            )
+      if (($state == $HITS)
+          &&
+          (
+           /^<!-- res_extend.wm -->/i
+           ||
+           m!<a\s+href="/r\?ext24"!i
+           ||
+           m!<a\s+href="http://jump.altavista.com/rlweb_ebay.go!i
+          )
+         )
         {
         ($hit, $raw) = $self->save_old_hit($hit, $raw);
         $state = $TRAILER;
         print STDERR "PARSE(13:INHIT->HITS): end hit.\n" if ($self->{_debug} >= 2);
         }
-      elsif ($hits_found && ($state == $TRAILER || $state == $HITS) && /<a[^>]+href="([^"]+)".*\&gt;\&gt;/i) { # "
+      if ($hits_found &&
+          (($state == $TRAILER)
+           ||
+           ($state == $HITS))
+          &&
+          m/<a[^>]+href="([^"]+)"[^>]*>[^>]+&gt;&gt;/i)
+        {
         # (above, note the trick $hits_found so we don't prematurely terminate.)
         # set up next page
-        my($relative_url) = $1;
+        my $relative_url = $1;
         # Actual line of input:
-        # &nbsp; <a href="/cgi-bin/query?pg=q&amp;nbq=50&amp;what=web&amp;text=yes&amp;fmt=d&amp;q=Martin+Thurn&stq=50" target="_self">[Next &gt;&gt;]</a> 
+        # &nbsp; <a href="/cgi-bin/query?pg=q&amp;nbq=50&amp;what=web&amp;text=yes&amp;fmt=d&amp;q=Martin+Thurn&stq=50" target="_self">[Next &gt;&gt;]</a>
 
         print STDERR "PARSE(15:->POST_NEXT): raw next_url is $relative_url\n" if ($self->{_debug} >= 2);
         # hack:  make sure fmt=d stays on news URLs
@@ -303,28 +386,41 @@ sub native_retrieve_some
         # altavista.com spit out double-encoded URLs!  I.e. they
         # contain &amp;amp; !!
         $relative_url =~ s!&amp;!&!g;
-        $self->{_next_url} = $HTTP::URI_CLASS->new_abs($relative_url, $self->{_base_url});
-        $state = $POST_NEXT;
-        print STDERR "PARSE(15:->POST_NEXT): cooked next_url is $n.\n" if ($self->{_debug} >= 2);
-    } elsif ($state == $HITS) {
+        my $sURLtry = $self->absurl($self->{_base_url}, $relative_url);
+        printf(STDERR "PARSE(15:->POST_NEXT): cooked next_url is $sURLtry.\n") if ($self->{_debug} >= 2);
+        if ($sURLtry =~ m!$self->{_options}->{search_host}$self->{_options}->{search_path}!)
+          {
+          # This really is the next URL.
+          $self->{_next_url} = $sURLtry;
+          $state = $POST_NEXT;
+          } # if
+        } # if
+
+      if (0 && ($state == $HITS))
+        {
         # other random stuff in a hit---accumulate it
         $raw .= $_;
         print STDERR "PARSE(14:INHIT): no match.\n" if ($self->{_debug} >= 2);
-            print STDERR ' 'x 12, "$_\n" if ($self->{_debug} >= 3);
-    } else {
+        print STDERR ' 'x 12, "$_\n" if ($self->{_debug} >= 3);
+        }
+      else
+        {
         # accumulate raw
         $raw .= $_;
         # print STDERR "PARSE(RAW): $_\n" if ($self->{_debug} >= 3);
-    };
-    };
-    if ($state != $POST_NEXT) {
-    # end, no other pages (missed ``next'' tag)
-    if ($state == $HITS) {
+        }
+      }
+
+    if ($state != $POST_NEXT)
+      {
+      # end, no other pages (missed ``next'' tag)
+      if ($state == $HITS)
+        {
         $self->begin_new_hit($hit, $raw);   # save old one
         print STDERR "PARSE: never got to TRAILER.\n" if ($self->{_debug} >= 2);
-    };
-    $self->{_next_url} = undef;
-    };
+        }
+      $self->{_next_url} = undef;
+      }
 
     # sleep so as to not overload altavista
     $self->user_agent_delay if (defined($self->{_next_url}));
@@ -334,3 +430,10 @@ sub native_retrieve_some
 
 1;
 
+__END__
+
+advanced search results:
+http://www.altavista.com/web/results?pg=aq&avkw=qtrp&aqmode=s&aqa=&aqp=&aqo=martin+thurn&aqn=&aqb=&aqs=&kgs=0&kls=0&dt=tmperiod&d2=0&d0=&d1=&filetype=&rc=dmn&swd=&lh=&nbq=50
+
+gui query results:
+http://www.altavista.com/web/results?q=Rhonda+Thurn&kgs=0&kls=0&avkw=qtrp
